@@ -17,26 +17,15 @@ import errno
 import json
 import os
 import time
-from swift import gettext_ as _
+from resource import getpagesize
 
 from swift import __version__ as swiftver
+from swift import gettext_ as _
+from swift.common.constraints import check_mount
 from swift.common.storage_policy import POLICIES
 from swift.common.swob import Request, Response
 from swift.common.utils import get_logger, config_true_value, \
-    SWIFT_CONF_FILE
-from swift.common.constraints import check_mount
-from resource import getpagesize
-from hashlib import md5
-
-
-MD5_BLOCK_READ_BYTES = 4096
-
-
-def _hash_for_ringfile(f):
-    md5sum = md5()
-    for block in iter(lambda: f.read(MD5_BLOCK_READ_BYTES), ''):
-        md5sum.update(block)
-    return md5sum.hexdigest()
+    SWIFT_CONF_FILE, md5_hash_for_file
 
 
 class ReconMiddleware(object):
@@ -256,35 +245,27 @@ class ReconMiddleware(object):
                                 'size': '', 'used': '', 'avail': ''})
         return devices
 
-    def get_ring_md5(self, openr=open):
+    def get_ring_md5(self):
         """get all ring md5sum's"""
         sums = {}
         for ringfile in self.rings:
             if os.path.exists(ringfile):
                 try:
-                    with openr(ringfile, 'rb') as f:
-                        sums[ringfile] = _hash_for_ringfile(f)
+                    sums[ringfile] = md5_hash_for_file(ringfile)
                 except IOError as err:
                     sums[ringfile] = None
                     if err.errno != errno.ENOENT:
                         self.logger.exception(_('Error reading ringfile'))
         return sums
 
-    def get_swift_conf_md5(self, openr=open):
+    def get_swift_conf_md5(self):
         """get md5 of swift.conf"""
-        md5sum = md5()
+        hexsum = None
         try:
-            with openr(SWIFT_CONF_FILE, 'r') as fh:
-                chunk = fh.read(4096)
-                while chunk:
-                    md5sum.update(chunk)
-                    chunk = fh.read(4096)
+            hexsum = md5_hash_for_file(SWIFT_CONF_FILE)
         except IOError as err:
             if err.errno != errno.ENOENT:
                 self.logger.exception(_('Error reading swift.conf'))
-            hexsum = None
-        else:
-            hexsum = md5sum.hexdigest()
         return {SWIFT_CONF_FILE: hexsum}
 
     def get_quarantine_count(self):
