@@ -40,7 +40,7 @@ from six.moves.http_client import HTTPException
 from swift.common.middleware.memcache import MemcacheMiddleware
 from swift.common.storage_policy import parse_storage_policies, PolicyError
 
-from test import get_config
+from test import get_config, listen_zero
 from test.functional.swift_test_client import Account, Connection, Container, \
     ResponseError
 # This has the side effect of mocking out the xattr module so that unit tests
@@ -259,7 +259,7 @@ def _in_process_setup_ring(swift_conf, conf_src_dir, testdir):
             device = 'sd%c1' % chr(len(obj_sockets) + ord('a'))
             utils.mkdirs(os.path.join(_testdir, 'sda1'))
             utils.mkdirs(os.path.join(_testdir, 'sda1', 'tmp'))
-            obj_socket = eventlet.listen(('localhost', 0))
+            obj_socket = listen_zero()
             obj_sockets.append(obj_socket)
             dev['port'] = obj_socket.getsockname()[1]
             dev['ip'] = '127.0.0.1'
@@ -270,7 +270,7 @@ def _in_process_setup_ring(swift_conf, conf_src_dir, testdir):
     else:
         # make default test ring, 2 replicas, 4 partitions, 2 devices
         _info('No source object ring file, creating 2rep/4part/2dev ring')
-        obj_sockets = [eventlet.listen(('localhost', 0)) for _ in (0, 1)]
+        obj_sockets = [listen_zero() for _ in (0, 1)]
         ring_data = ring.RingData(
             [[0, 1, 0, 1], [1, 0, 1, 0]],
             [{'id': 0, 'zone': 0, 'device': 'sda1', 'ip': '127.0.0.1',
@@ -416,7 +416,7 @@ def in_process_setup(the_object_server=object_server):
     # We create the proxy server listening socket to get its port number so
     # that we can add it as the "auth_port" value for the functional test
     # clients.
-    prolis = eventlet.listen(('localhost', 0))
+    prolis = listen_zero()
     _test_socks.append(prolis)
 
     # The following set of configuration values is used both for the
@@ -473,10 +473,10 @@ def in_process_setup(the_object_server=object_server):
         config['object_post_as_copy'] = str(object_post_as_copy)
         _debug('Setting object_post_as_copy to %r' % object_post_as_copy)
 
-    acc1lis = eventlet.listen(('localhost', 0))
-    acc2lis = eventlet.listen(('localhost', 0))
-    con1lis = eventlet.listen(('localhost', 0))
-    con2lis = eventlet.listen(('localhost', 0))
+    acc1lis = listen_zero()
+    acc2lis = listen_zero()
+    con1lis = listen_zero()
+    con2lis = listen_zero()
     _test_socks += [acc1lis, acc2lis, con1lis, con2lis] + obj_sockets
 
     account_ring_path = os.path.join(_testdir, 'account.ring.gz')
