@@ -23,9 +23,8 @@ from unittest2 import SkipTest
 
 import test.functional as tf
 from test.functional import cluster_info
-from test.functional.tests import Utils, Base, Base2
-from test.functional.swift_test_client import Account, Connection, \
-    ResponseError
+from test.functional.tests import Utils, Base, Base2, BaseEnv
+from test.functional.swift_test_client import Connection, ResponseError
 
 
 def setUpModule():
@@ -36,7 +35,7 @@ def tearDownModule():
     tf.teardown_package()
 
 
-class TestSloEnv(object):
+class TestSloEnv(BaseEnv):
     slo_enabled = None  # tri-state: None initially, then True/False
 
     @classmethod
@@ -58,8 +57,13 @@ class TestSloEnv(object):
 
     @classmethod
     def setUp(cls):
-        cls.conn = Connection(tf.config)
-        cls.conn.authenticate()
+        if cls.slo_enabled is None:
+            cls.slo_enabled = 'slo' in cluster_info
+            if not cls.slo_enabled:
+                return
+
+        super(TestSloEnv, cls).setUp()
+
         config2 = deepcopy(tf.config)
         config2['account'] = tf.config['account2']
         config2['username'] = tf.config['username2']
@@ -74,15 +78,6 @@ class TestSloEnv(object):
         cls.conn3 = Connection(config3)
         cls.conn3.authenticate()
 
-        if cls.slo_enabled is None:
-            cls.slo_enabled = 'slo' in cluster_info
-            if not cls.slo_enabled:
-                return
-
-        cls.account = Account(cls.conn, tf.config.get('account',
-                                                      tf.config['username']))
-        cls.account.delete_containers()
-
         cls.container = cls.account.container(Utils.create_name())
         cls.container2 = cls.account.container(Utils.create_name())
 
@@ -214,7 +209,6 @@ class TestSloEnv(object):
 
 class TestSlo(Base):
     env = TestSloEnv
-    set_up = False
 
     def setUp(self):
         super(TestSlo, self).setUp()
@@ -360,6 +354,33 @@ class TestSlo(Base):
         self.assertEqual('b', file_contents[-2])
         self.assertEqual('c', file_contents[-1])
 
+    def test_slo_ranged_get_half_open_on_right(self):
+        file_item = self.env.container.file('manifest-abcde')
+        file_contents = file_item.read(
+            hdrs={"Range": "bytes=1048571-"})
+        grouped_file_contents = [
+            (char, sum(1 for _char in grp))
+            for char, grp in itertools.groupby(file_contents)]
+        self.assertEqual([
+            ('a', 5),
+            ('b', 1048576),
+            ('c', 1048576),
+            ('d', 1048576),
+            ('e', 1)
+        ], grouped_file_contents)
+
+    def test_slo_ranged_get_half_open_on_left(self):
+        file_item = self.env.container.file('manifest-abcde')
+        file_contents = file_item.read(
+            hdrs={"Range": "bytes=-123456"})
+        grouped_file_contents = [
+            (char, sum(1 for _char in grp))
+            for char, grp in itertools.groupby(file_contents)]
+        self.assertEqual([
+            ('d', 123455),
+            ('e', 1),
+        ], grouped_file_contents)
+
     def test_slo_multi_ranged_get(self):
         file_item = self.env.container.file('manifest-abcde')
         file_contents = file_item.read(
@@ -451,6 +472,35 @@ class TestSlo(Base):
         else:
             self.fail("Expected ResponseError but didn't get it")
 
+    def test_slo_client_etag_mismatch(self):
+        file_item = self.env.container.file("manifest-a-mismatch-etag")
+        try:
+            file_item.write(
+                json.dumps([{
+                    'size_bytes': 1024 * 1024,
+                    'etag': hashlib.md5('a' * 1024 * 1024).hexdigest(),
+                    'path': '/%s/%s' % (self.env.container.name, 'seg_a')}]),
+                parms={'multipart-manifest': 'put'},
+                hdrs={'Etag': 'NOTetagofthesegments'})
+        except ResponseError as err:
+            self.assertEqual(422, err.status)
+
+    def test_slo_client_etag(self):
+        file_item = self.env.container.file("manifest-a-b-etag")
+        etag_a = hashlib.md5('a' * 1024 * 1024).hexdigest()
+        etag_b = hashlib.md5('b' * 1024 * 1024).hexdigest()
+        file_item.write(
+            json.dumps([{
+                'size_bytes': 1024 * 1024,
+                'etag': etag_a,
+                'path': '/%s/%s' % (self.env.container.name, 'seg_a')}, {
+                'size_bytes': 1024 * 1024,
+                'etag': etag_b,
+                'path': '/%s/%s' % (self.env.container.name, 'seg_b')}]),
+            parms={'multipart-manifest': 'put'},
+            hdrs={'Etag': hashlib.md5(etag_a + etag_b).hexdigest()})
+        self.assert_status(201)
+
     def test_slo_unspecified_etag(self):
         file_item = self.env.container.file("manifest-a-unspecified-etag")
         file_item.write(
@@ -963,4 +1013,4 @@ class TestSlo(Base):
 
 
 class TestSloUTF8(Base2, TestSlo):
-    set_up = False
+    pass
