@@ -49,9 +49,6 @@ class Test(ReplProbeTest):
                                              self.container_name)
         self.int_client = self.make_internal_client(object_post_as_copy=False)
 
-    def tearDown(self):
-        super(Test, self).tearDown()
-
     def _get_object_info(self, account, container, obj, number):
         obj_conf = self.configs['object-server']
         config_path = obj_conf[number]
@@ -79,7 +76,7 @@ class Test(ReplProbeTest):
                                            self.object_name, i)
             if info_i:
                 obj_info.append(info_i)
-        self.assertTrue(len(obj_info) > 1)
+        self.assertGreater(len(obj_info), 1)
         for other in obj_info[1:]:
             self.assertDictEqual(obj_info[0], other)
 
@@ -124,7 +121,7 @@ class Test(ReplProbeTest):
             info_i = self._get_db_info(self.account, self.container_name, i)
             if info_i:
                 db_info.append(info_i)
-        self.assertTrue(len(db_info) > 1)
+        self.assertGreater(len(db_info), 1)
         for other in db_info[1:]:
             self.assertEqual(db_info[0]['hash'], other['hash'],
                              'Container db hash mismatch: %s != %s'
@@ -278,7 +275,7 @@ class Test(ReplProbeTest):
         self._put_object(headers=sysmeta)
         metadata = self._get_object_metadata()
         for key in sysmeta:
-            self.assertTrue(key in metadata)
+            self.assertIn(key, metadata)
             self.assertEqual(metadata[key], sysmeta[key])
         self.brain.start_primary_half()
         self.container_brain.start_primary_half()
@@ -289,15 +286,15 @@ class Test(ReplProbeTest):
         self._put_object(headers=sysmeta2)
         metadata = self._get_object_metadata()
         for key in sysmeta2:
-            self.assertTrue(key in metadata)
+            self.assertIn(key, metadata)
             self.assertEqual(metadata[key], sysmeta2[key])
         self._post_object(usermeta)
         metadata = self._get_object_metadata()
         for key in usermeta:
-            self.assertTrue(key in metadata)
+            self.assertIn(key, metadata)
             self.assertEqual(metadata[key], usermeta[key])
         for key in sysmeta2:
-            self.assertTrue(key in metadata)
+            self.assertIn(key, metadata)
             self.assertEqual(metadata[key], sysmeta2[key])
 
         self.brain.start_handoff_half()
@@ -311,10 +308,10 @@ class Test(ReplProbeTest):
         self.container_brain.stop_primary_half()
         metadata = self._get_object_metadata()
         for key in usermeta:
-            self.assertTrue(key in metadata)
+            self.assertIn(key, metadata)
             self.assertEqual(metadata[key], usermeta[key])
         for key in sysmeta2.keys():
-            self.assertTrue(key in metadata, key)
+            self.assertIn(key, metadata, key)
             self.assertEqual(metadata[key], sysmeta2[key])
         self.brain.start_primary_half()
         self.container_brain.start_primary_half()
@@ -324,10 +321,10 @@ class Test(ReplProbeTest):
         self.container_brain.stop_handoff_half()
         metadata = self._get_object_metadata()
         for key in usermeta:
-            self.assertTrue(key in metadata)
+            self.assertIn(key, metadata)
             self.assertEqual(metadata[key], usermeta[key])
         for key in sysmeta2.keys():
-            self.assertTrue(key in metadata, key)
+            self.assertIn(key, metadata, key)
             self.assertEqual(metadata[key], sysmeta2[key])
         self.brain.start_handoff_half()
         self.container_brain.start_handoff_half()
@@ -339,6 +336,8 @@ class Test(ReplProbeTest):
     def test_sysmeta_after_replication_with_subsequent_post(self):
         sysmeta = {'x-object-sysmeta-foo': 'sysmeta-foo'}
         usermeta = {'x-object-meta-bar': 'meta-bar'}
+        transient_sysmeta = {
+            'x-object-transient-sysmeta-bar': 'transient-sysmeta-bar'}
         self.brain.put_container(policy_index=int(self.policy))
         # put object
         self._put_object()
@@ -348,7 +347,7 @@ class Test(ReplProbeTest):
         self._put_object(headers=sysmeta)
         metadata = self._get_object_metadata()
         for key in sysmeta:
-            self.assertTrue(key in metadata)
+            self.assertIn(key, metadata)
             self.assertEqual(metadata[key], sysmeta[key])
         self.brain.start_primary_half()
         self.container_brain.start_primary_half()
@@ -356,13 +355,15 @@ class Test(ReplProbeTest):
         # post some user meta to second server subset
         self.brain.stop_handoff_half()
         self.container_brain.stop_handoff_half()
-        self._post_object(usermeta)
+        user_and_transient_sysmeta = dict(usermeta)
+        user_and_transient_sysmeta.update(transient_sysmeta)
+        self._post_object(user_and_transient_sysmeta)
         metadata = self._get_object_metadata()
-        for key in usermeta:
-            self.assertTrue(key in metadata)
-            self.assertEqual(metadata[key], usermeta[key])
+        for key in user_and_transient_sysmeta:
+            self.assertIn(key, metadata)
+            self.assertEqual(metadata[key], user_and_transient_sysmeta[key])
         for key in sysmeta:
-            self.assertFalse(key in metadata)
+            self.assertNotIn(key, metadata)
         self.brain.start_handoff_half()
         self.container_brain.start_handoff_half()
 
@@ -376,8 +377,9 @@ class Test(ReplProbeTest):
         metadata = self._get_object_metadata()
         expected = dict(sysmeta)
         expected.update(usermeta)
+        expected.update(transient_sysmeta)
         for key in expected.keys():
-            self.assertTrue(key in metadata, key)
+            self.assertIn(key, metadata, key)
             self.assertEqual(metadata[key], expected[key])
         self.brain.start_primary_half()
         self.container_brain.start_primary_half()
@@ -387,7 +389,7 @@ class Test(ReplProbeTest):
         self.container_brain.stop_handoff_half()
         metadata = self._get_object_metadata()
         for key in expected.keys():
-            self.assertTrue(key in metadata, key)
+            self.assertIn(key, metadata, key)
             self.assertEqual(metadata[key], expected[key])
         self.brain.start_handoff_half()
         self.container_brain.start_handoff_half()
@@ -399,6 +401,8 @@ class Test(ReplProbeTest):
     def test_sysmeta_after_replication_with_prior_post(self):
         sysmeta = {'x-object-sysmeta-foo': 'sysmeta-foo'}
         usermeta = {'x-object-meta-bar': 'meta-bar'}
+        transient_sysmeta = {
+            'x-object-transient-sysmeta-bar': 'transient-sysmeta-bar'}
         self.brain.put_container(policy_index=int(self.policy))
         # put object
         self._put_object()
@@ -406,11 +410,13 @@ class Test(ReplProbeTest):
         # put user meta to first server subset
         self.brain.stop_handoff_half()
         self.container_brain.stop_handoff_half()
-        self._post_object(headers=usermeta)
+        user_and_transient_sysmeta = dict(usermeta)
+        user_and_transient_sysmeta.update(transient_sysmeta)
+        self._post_object(user_and_transient_sysmeta)
         metadata = self._get_object_metadata()
-        for key in usermeta:
-            self.assertTrue(key in metadata)
-            self.assertEqual(metadata[key], usermeta[key])
+        for key in user_and_transient_sysmeta:
+            self.assertIn(key, metadata)
+            self.assertEqual(metadata[key], user_and_transient_sysmeta[key])
         self.brain.start_handoff_half()
         self.container_brain.start_handoff_half()
 
@@ -420,7 +426,7 @@ class Test(ReplProbeTest):
         self._put_object(headers=sysmeta)
         metadata = self._get_object_metadata()
         for key in sysmeta:
-            self.assertTrue(key in metadata)
+            self.assertIn(key, metadata)
             self.assertEqual(metadata[key], sysmeta[key])
         self.brain.start_primary_half()
         self.container_brain.start_primary_half()
@@ -434,10 +440,10 @@ class Test(ReplProbeTest):
         self.container_brain.stop_primary_half()
         metadata = self._get_object_metadata()
         for key in sysmeta:
-            self.assertTrue(key in metadata)
+            self.assertIn(key, metadata)
             self.assertEqual(metadata[key], sysmeta[key])
-        for key in usermeta:
-            self.assertFalse(key in metadata)
+        for key in user_and_transient_sysmeta:
+            self.assertNotIn(key, metadata)
         self.brain.start_primary_half()
         self.container_brain.start_primary_half()
 
@@ -447,10 +453,10 @@ class Test(ReplProbeTest):
         self.container_brain.stop_handoff_half()
         metadata = self._get_object_metadata()
         for key in sysmeta:
-            self.assertTrue(key in metadata)
+            self.assertIn(key, metadata)
             self.assertEqual(metadata[key], sysmeta[key])
-        for key in usermeta:
-            self.assertFalse(key in metadata)
+        for key in user_and_transient_sysmeta:
+            self.assertNotIn(key, metadata)
         self.brain.start_handoff_half()
         self.container_brain.start_handoff_half()
 
