@@ -23,7 +23,7 @@ import mock
 
 from swift.common import swob
 from swift.common.middleware import container_sync
-from swift.proxy.controllers.base import _get_cache_key
+from swift.proxy.controllers.base import get_cache_key
 from swift.proxy.controllers.info import InfoController
 
 from test.unit import FakeLogger
@@ -205,16 +205,17 @@ cluster_dfw1 = http://dfw1.host/v1/
     def test_invalid_sig(self):
         req = swob.Request.blank(
             '/v1/a/c', headers={'x-container-sync-auth': 'US nonce sig'})
-        req.environ[_get_cache_key('a', 'c')[1]] = {'sync_key': 'abc'}
+        infocache = req.environ.setdefault('swift.infocache', {})
+        infocache[get_cache_key('a', 'c')] = {'sync_key': 'abc'}
         resp = req.get_response(self.sync)
         self.assertEqual(resp.status, '401 Unauthorized')
         self.assertEqual(
             resp.body,
             'X-Container-Sync-Auth header not valid; contact cluster operator '
             'for support.')
-        self.assertTrue(
-            'cs:invalid-sig' in req.environ.get('swift.log_info'),
-            req.environ.get('swift.log_info'))
+        self.assertIn('cs:invalid-sig', req.environ.get('swift.log_info'))
+        self.assertNotIn('swift.authorize_override', req.environ)
+        self.assertNotIn('swift.slo_override', req.environ)
 
     def test_valid_sig(self):
         ts = '1455221706.726999_0123456789abcdef'
@@ -224,13 +225,16 @@ cluster_dfw1 = http://dfw1.host/v1/
         req = swob.Request.blank('/v1/a/c', headers={
             'x-container-sync-auth': 'US nonce ' + sig,
             'x-backend-inbound-x-timestamp': ts})
-        req.environ[_get_cache_key('a', 'c')[1]] = {'sync_key': 'abc'}
+        infocache = req.environ.setdefault('swift.infocache', {})
+        infocache[get_cache_key('a', 'c')] = {'sync_key': 'abc'}
         resp = req.get_response(self.sync)
         self.assertEqual(resp.status, '200 OK')
         self.assertEqual(resp.body, 'Response to Authorized Request')
         self.assertIn('cs:valid', req.environ.get('swift.log_info'))
         self.assertIn('X-Timestamp', resp.headers)
         self.assertEqual(ts, resp.headers['X-Timestamp'])
+        self.assertIn('swift.authorize_override', req.environ)
+        self.assertIn('swift.slo_override', req.environ)
 
     def test_valid_sig2(self):
         sig = self.sync.realms_conf.get_sig(
@@ -238,13 +242,14 @@ cluster_dfw1 = http://dfw1.host/v1/
             self.sync.realms_conf.key2('US'), 'abc')
         req = swob.Request.blank(
             '/v1/a/c', headers={'x-container-sync-auth': 'US nonce ' + sig})
-        req.environ[_get_cache_key('a', 'c')[1]] = {'sync_key': 'abc'}
+        infocache = req.environ.setdefault('swift.infocache', {})
+        infocache[get_cache_key('a', 'c')] = {'sync_key': 'abc'}
         resp = req.get_response(self.sync)
         self.assertEqual(resp.status, '200 OK')
         self.assertEqual(resp.body, 'Response to Authorized Request')
-        self.assertTrue(
-            'cs:valid' in req.environ.get('swift.log_info'),
-            req.environ.get('swift.log_info'))
+        self.assertIn('cs:valid', req.environ.get('swift.log_info'))
+        self.assertIn('swift.authorize_override', req.environ)
+        self.assertIn('swift.slo_override', req.environ)
 
     def test_info(self):
         req = swob.Request.blank('/info')
