@@ -150,13 +150,15 @@ class TestServerSideCopyMiddleware(unittest.TestCase):
         self.assertEqual(len(self.authorized), 1)
         self.assertRequestEqual(req, self.authorized[0])
 
-    def test_object_delete_pass_through(self):
-        self.app.register('DELETE', '/v1/a/c/o', swob.HTTPOk, {})
-        req = Request.blank('/v1/a/c/o', method='DELETE')
-        status, headers, body = self.call_ssc(req)
-        self.assertEqual(status, '200 OK')
-        self.assertEqual(len(self.authorized), 1)
-        self.assertRequestEqual(req, self.authorized[0])
+    def test_object_pass_through_methods(self):
+        for method in ['DELETE', 'GET', 'HEAD', 'REPLICATE']:
+            self.app.register(method, '/v1/a/c/o', swob.HTTPOk, {})
+            req = Request.blank('/v1/a/c/o', method=method)
+            status, headers, body = self.call_ssc(req)
+            self.assertEqual(status, '200 OK')
+            self.assertEqual(len(self.authorized), 1)
+            self.assertRequestEqual(req, self.authorized[0])
+            self.assertNotIn('swift.orig_req_method', req.environ)
 
     def test_POST_as_COPY_simple(self):
         self.app.register('GET', '/v1/a/c/o', swob.HTTPOk, {}, 'passed')
@@ -166,6 +168,8 @@ class TestServerSideCopyMiddleware(unittest.TestCase):
         self.assertEqual(status, '202 Accepted')
         self.assertEqual(len(self.authorized), 1)
         self.assertRequestEqual(req, self.authorized[0])
+        # For basic test cases, assert orig_req_method behavior
+        self.assertEqual(req.environ['swift.orig_req_method'], 'POST')
 
     def test_POST_as_COPY_201_return_202(self):
         self.app.register('GET', '/v1/a/c/o', swob.HTTPOk, {}, 'passed')
@@ -258,6 +262,8 @@ class TestServerSideCopyMiddleware(unittest.TestCase):
         self.assertEqual('/v1/a/c/o2', self.authorized[1].path)
         self.assertEqual(self.app.swift_sources[0], 'SSC')
         self.assertEqual(self.app.swift_sources[1], 'SSC')
+        # For basic test cases, assert orig_req_method behavior
+        self.assertNotIn('swift.orig_req_method', req.environ)
 
     def test_static_large_object_manifest(self):
         self.app.register('GET', '/v1/a/c/o', swob.HTTPOk,
@@ -663,6 +669,8 @@ class TestServerSideCopyMiddleware(unittest.TestCase):
             ('PUT', '/v1/a/c/o-copy')])
         self.assertIn('etag', self.app.headers[1])
         self.assertEqual(self.app.headers[1]['etag'], 'is sent')
+        # For basic test cases, assert orig_req_method behavior
+        self.assertEqual(req.environ['swift.orig_req_method'], 'COPY')
 
     def test_basic_DLO(self):
         self.app.register('GET', '/v1/a/c/o', swob.HTTPOk, {
@@ -1191,6 +1199,8 @@ class TestServerSideCopyMiddleware(unittest.TestCase):
         self.assertEqual(len(self.authorized), 1)
         self.assertEqual('OPTIONS', self.authorized[0].method)
         self.assertEqual('/v1/a/c/o', self.authorized[0].path)
+        # For basic test cases, assert orig_req_method behavior
+        self.assertNotIn('swift.orig_req_method', req.environ)
 
     def test_COPY_in_OPTIONS_response_CORS(self):
         self.app.register('OPTIONS', '/v1/a/c/o', swob.HTTPOk,
