@@ -411,6 +411,12 @@ class TestDloGetManifest(DloTestCase):
                                  headers={'Range': 'bytes=25-30'})
         status, headers, body = self.call_dlo(req)
         self.assertEqual(status, "416 Requested Range Not Satisfiable")
+        expected_headers = (
+            ('Accept-Ranges', 'bytes'),
+            ('Content-Range', 'bytes */25'),
+        )
+        for header_value_pair in expected_headers:
+            self.assertIn(header_value_pair, headers)
 
     def test_get_range_many_segments_satisfiable(self):
         req = swob.Request.blank('/v1/AUTH_test/mancon/manifest-many-segments',
@@ -803,107 +809,6 @@ class TestDloGetManifest(DloTestCase):
         self.assertTrue(auth_got_called[0] > 1)
 
 
-def fake_start_response(*args, **kwargs):
-    pass
-
-
-class TestDloCopyHook(DloTestCase):
-    def setUp(self):
-        super(TestDloCopyHook, self).setUp()
-
-        self.app.register(
-            'GET', '/v1/AUTH_test/c/o1', swob.HTTPOk,
-            {'Content-Length': '10', 'Etag': 'o1-etag'},
-            "aaaaaaaaaa")
-        self.app.register(
-            'GET', '/v1/AUTH_test/c/o2', swob.HTTPOk,
-            {'Content-Length': '10', 'Etag': 'o2-etag'},
-            "bbbbbbbbbb")
-        self.app.register(
-            'GET', '/v1/AUTH_test/c/man',
-            swob.HTTPOk, {'X-Object-Manifest': 'c/o'},
-            "manifest-contents")
-
-        lm = '2013-11-22T02:42:13.781760'
-        ct = 'application/octet-stream'
-        segs = [{"hash": "o1-etag", "bytes": 10, "name": "o1",
-                 "last_modified": lm, "content_type": ct},
-                {"hash": "o2-etag", "bytes": 5, "name": "o2",
-                 "last_modified": lm, "content_type": ct}]
-
-        self.app.register(
-            'GET', '/v1/AUTH_test/c?format=json&prefix=o',
-            swob.HTTPOk, {'Content-Type': 'application/json; charset=utf-8'},
-            json.dumps(segs))
-
-        copy_hook = [None]
-
-        # slip this guy in there to pull out the hook
-        def extract_copy_hook(env, sr):
-            copy_hook[0] = env.get('swift.copy_hook')
-            return self.app(env, sr)
-
-        self.dlo = dlo.filter_factory({})(extract_copy_hook)
-
-        req = swob.Request.blank('/v1/AUTH_test/c/o1',
-                                 environ={'REQUEST_METHOD': 'GET'})
-        self.dlo(req.environ, fake_start_response)
-        self.copy_hook = copy_hook[0]
-
-        self.assertTrue(self.copy_hook is not None)  # sanity check
-
-    def test_copy_hook_passthrough(self):
-        source_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'GET'})
-        sink_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'PUT'})
-        source_resp = swob.Response(request=source_req, status=200)
-
-        # no X-Object-Manifest header, so do nothing
-        modified_resp = self.copy_hook(source_req, source_resp, sink_req)
-        self.assertTrue(modified_resp is source_resp)
-
-    def test_copy_hook_manifest(self):
-        source_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'GET'})
-        sink_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'PUT'})
-        source_resp = swob.Response(
-            request=source_req, status=200,
-            headers={"X-Object-Manifest": "c/o"},
-            app_iter=["manifest"])
-
-        # it's a manifest, so copy the segments to make a normal object
-        modified_resp = self.copy_hook(source_req, source_resp, sink_req)
-        self.assertTrue(modified_resp is not source_resp)
-        self.assertEqual(modified_resp.etag,
-                         hashlib.md5("o1-etago2-etag").hexdigest())
-        self.assertEqual(sink_req.headers.get('X-Object-Manifest'), None)
-
-    def test_copy_hook_manifest_with_multipart_manifest_get(self):
-        source_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'GET',
-                     'QUERY_STRING': 'multipart-manifest=get'})
-        sink_req = swob.Request.blank(
-            '/v1/AUTH_test/c/man',
-            environ={'REQUEST_METHOD': 'PUT'})
-        source_resp = swob.Response(
-            request=source_req, status=200,
-            headers={"X-Object-Manifest": "c/o"},
-            app_iter=["manifest"])
-
-        # make sure the sink request (the backend PUT) gets X-Object-Manifest
-        # on it, but that's all
-        modified_resp = self.copy_hook(source_req, source_resp, sink_req)
-        self.assertTrue(modified_resp is source_resp)
-        self.assertEqual(sink_req.headers.get('X-Object-Manifest'), 'c/o')
-
-
 class TestDloConfiguration(unittest.TestCase):
     """
     For backwards compatibility, we will read a couple of values out of the
