@@ -21,7 +21,6 @@ from posix import stat_result, statvfs_result
 from shutil import rmtree
 import unittest
 from unittest import TestCase
-from six import StringIO
 
 from swift import __version__ as swiftver
 from swift.common import ring, utils
@@ -232,12 +231,12 @@ class TestReconSuccess(TestCase):
         self.app._from_recon_cache = self.fakecache.fake_from_recon_cache
         self.frecon = FakeRecon()
 
-        # replace hash md5 implementation of the _hash_for_ringfile function
-        mock_hash_for_ringfile = mock.patch(
-            'swift.common.middleware.recon._hash_for_ringfile',
-            lambda f: 'hash-' + os.path.basename(f.name))
-        self.addCleanup(mock_hash_for_ringfile.stop)
-        mock_hash_for_ringfile.start()
+        # replace hash md5 implementation of the md5_hash_for_file function
+        mock_hash_for_file = mock.patch(
+            'swift.common.middleware.recon.md5_hash_for_file',
+            lambda f, **kwargs: 'hash-' + os.path.basename(f))
+        self.addCleanup(mock_hash_for_file.stop)
+        mock_hash_for_file.start()
 
         self.ring_part_shift = 5
         self.ring_devs = [{'id': 0, 'zone': 0, 'weight': 1.0,
@@ -347,14 +346,12 @@ class TestReconSuccess(TestCase):
         # still produce a ringmd5 entry with a None for the hash. Note that
         # this is different than if an expected ring file simply doesn't exist,
         # in which case it is excluded altogether from the ringmd5 response.
-
-        def fake_open(fn, fmode):
-            raise IOError
-
         expt_out = {'%s/account.ring.gz' % self.tempdir: None,
                     '%s/container.ring.gz' % self.tempdir: None,
                     '%s/object.ring.gz' % self.tempdir: None}
-        ringmd5 = self.app.get_ring_md5(openr=fake_open)
+        with mock.patch('swift.common.middleware.recon.md5_hash_for_file',
+                        side_effect=IOError):
+            ringmd5 = self.app.get_ring_md5()
         self.assertEqual(sorted(ringmd5.items()),
                          sorted(expt_out.items()))
 
@@ -362,30 +359,30 @@ class TestReconSuccess(TestCase):
         # Ring files that are present but produce an IOError on read will
         # show a None hash, but if they can be read later their hash
         # should become available in the ringmd5 response.
-
-        def fake_open(fn, fmode):
-            raise IOError
-
         expt_out = {'%s/account.ring.gz' % self.tempdir: None,
                     '%s/container.ring.gz' % self.tempdir: None,
                     '%s/object.ring.gz' % self.tempdir: None}
-        ringmd5 = self.app.get_ring_md5(openr=fake_open)
+        with mock.patch('swift.common.middleware.recon.md5_hash_for_file',
+                        side_effect=IOError):
+            ringmd5 = self.app.get_ring_md5()
         self.assertEqual(sorted(ringmd5.items()),
                          sorted(expt_out.items()))
 
         # If we fix a ring and it can be read again, its hash should then
         # appear using the same app instance
-        def fake_open_objonly(fn, fmode):
+        def fake_hash_for_file(fn):
             if 'object' not in fn:
                 raise IOError
-            return open(fn, fmode)
+            return 'hash-' + os.path.basename(fn)
 
         expt_out = {'%s/account.ring.gz' % self.tempdir: None,
                     '%s/container.ring.gz' % self.tempdir: None,
                     '%s/object.ring.gz' % self.tempdir:
                     'hash-object.ring.gz'}
 
-        ringmd5 = self.app.get_ring_md5(openr=fake_open_objonly)
+        with mock.patch('swift.common.middleware.recon.md5_hash_for_file',
+                        fake_hash_for_file):
+            ringmd5 = self.app.get_ring_md5()
         self.assertEqual(sorted(ringmd5.items()),
                          sorted(expt_out.items()))
 
@@ -1414,90 +1411,11 @@ class TestReconMiddleware(unittest.TestCase):
 
     def test_get_swift_conf_md5_fail(self):
         """Test get_swift_conf_md5 failure by failing file open"""
-        resp = self.real_app_get_swift_conf_md5(fail_io_open)
+        with mock.patch('swift.common.middleware.recon.md5_hash_for_file',
+                        side_effect=IOError):
+            resp = self.real_app_get_swift_conf_md5()
         self.assertIsNone(resp['/etc/swift/swift.conf'])
 
 
-class TestReconUtilityFunctions(unittest.TestCase):
-
-    def test_hash_for_ringfile_on_filelike_smallish(self):
-        stub_data = 'some data'
-        stub_filelike = StringIO(stub_data)
-        with mock.patch('swift.common.middleware.recon.md5') as mock_md5:
-            mock_hasher = mock_md5.return_value
-            rv = recon._hash_for_ringfile(stub_filelike)
-        self.assertTrue(mock_hasher.hexdigest.called)
-        self.assertEqual(rv, mock_hasher.hexdigest.return_value)
-        self.assertEqual([mock.call(stub_data)],
-                         mock_hasher.update.call_args_list)
-
-    def test_hash_for_ringfile_on_filelike_big(self):
-        num_blocks = 10
-        block_size = recon.MD5_BLOCK_READ_BYTES
-        truncate = 523
-        start_char = ord('a')
-        expected_blocks = [chr(i) * block_size
-                           for i in range(start_char, start_char + num_blocks)]
-        full_data = ''.join(expected_blocks)
-        trimmed_data = full_data[:-truncate]
-        # sanity
-        self.assertEqual(len(trimmed_data), block_size * num_blocks - truncate)
-        stub_filelike = StringIO(trimmed_data)
-        with mock.patch('swift.common.middleware.recon.md5') as mock_md5:
-            mock_hasher = mock_md5.return_value
-            rv = recon._hash_for_ringfile(stub_filelike)
-        self.assertTrue(mock_hasher.hexdigest.called)
-        self.assertEqual(rv, mock_hasher.hexdigest.return_value)
-        self.assertEqual(num_blocks, len(mock_hasher.update.call_args_list))
-        expected_block = 'a' * block_size
-        found_blocks = []
-        for i, (expected_block, call) in enumerate(zip(
-                expected_blocks, mock_hasher.update.call_args_list)):
-            args, kwargs = call
-            self.assertEqual(kwargs, {})
-            self.assertEqual(1, len(args))
-            block = args[0]
-            if i < num_blocks - 1:
-                self.assertEqual(block, expected_block)
-            else:
-                self.assertEqual(block, expected_block[:-truncate])
-            found_blocks.append(block)
-        self.assertEqual(''.join(found_blocks), trimmed_data)
-
-    def test_hash_for_ringfile_on_filelike_empty(self):
-        stub_filelike = StringIO('')
-        with mock.patch('swift.common.middleware.recon.md5') as mock_md5:
-            mock_hasher = mock_md5.return_value
-            rv = recon._hash_for_ringfile(stub_filelike)
-        self.assertTrue(mock_hasher.hexdigest.called)
-        self.assertEqual(rv, mock_hasher.hexdigest.return_value)
-        self.assertEqual([], mock_hasher.update.call_args_list)
-
-    def test_hash_for_ringfile_on_filelike_brittle(self):
-        data_to_expected_hash = {
-            '': 'd41d8cd98f00b204e9800998ecf8427e',
-            'some data': '1e50210a0202497fb79bc38b6ade6c34',
-            ('a' * 4096 * 10)[:-523]: '06a41551609656c85f14f659055dc6d3',
-        }
-        # unlike some other places where the concrete implementation really
-        # matters for backwards compatibility these brittle tests are probably
-        # not needed or justified, if a future maintainer rips them out later
-        # they're probably doing the right thing
-        failures = []
-        for stub_data, expected_hash in data_to_expected_hash.items():
-            rv = recon._hash_for_ringfile(StringIO(stub_data))
-            try:
-                self.assertEqual(expected_hash, rv)
-            except AssertionError:
-                trim_cap = 80
-                if len(stub_data) > trim_cap:
-                    stub_data = '%s...<truncated>' % stub_data[:trim_cap]
-                failures.append('hash for %r was %s instead of expected %s' % (
-                    stub_data, rv, expected_hash))
-        if failures:
-            self.fail('Some data did not compute expected hash:\n' +
-                      '\n'.join(failures))
-
-
 if __name__ == '__main__':
     unittest.main()
