@@ -17,6 +17,8 @@ import json
 import unittest
 import mock
 
+from six.moves.urllib.parse import urlparse
+
 from swift.common.swob import Request, Response, HTTPUnauthorized
 from swift.common.middleware import staticweb
 
@@ -41,7 +43,8 @@ meta_map = {
                     'web-error': 'error.html'}},
     'c6b': {'meta': {'web-listings': 't',
                      'web-listings-label': 'foo'}},
-    'c7': {'meta': {'web-listings': 'f'}},
+    'c7': {'meta': {'web-listings': 'f',
+                    'web-error': 'error.html'}},
     'c8': {'meta': {'web-error': 'error.html',
                     'web-listings': 't',
                     'web-listings-css':
@@ -202,6 +205,16 @@ class FakeApp(object):
             '''.strip())(env, start_response)
         elif env['PATH_INFO'] in ('/v1/a/c7', '/v1/a/c7/'):
             return self.listing(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c7/404error.html':
+            return Response(status='404 Not Found')(env, start_response)
+        elif env['PATH_INFO'] == '/v1/a/c7/401error.html':
+            return Response(status='200 Ok', body='''
+<html>
+    <body style="background: #000000; color: #ffaaaa">
+        <p>Hey, you're not authorized to see this!</p>
+    </body>
+</html>
+            '''.strip())(env, start_response)
         elif env['PATH_INFO'] in ('/v1/a/c8', '/v1/a/c8/'):
             return self.listing(env, start_response)
         elif env['PATH_INFO'] == '/v1/a/c8/subdir/':
@@ -543,8 +556,8 @@ class TestStaticWeb(unittest.TestCase):
         self.assertEqual(resp.status_int, 200)
         self.assertTrue('Listing of /v1/a/c3/subdir/' in resp.body)
         self.assertTrue('</style>' in resp.body)
-        self.assertTrue('<link' not in resp.body)
-        self.assertTrue('listing.css' not in resp.body)
+        self.assertNotIn('<link', resp.body)
+        self.assertNotIn('listing.css', resp.body)
 
     def test_container3subdirx(self):
         resp = Request.blank(
@@ -565,7 +578,7 @@ class TestStaticWeb(unittest.TestCase):
         resp = Request.blank(
             '/v1/a/c3/unknown').get_response(self.test_staticweb)
         self.assertEqual(resp.status_int, 404)
-        self.assertTrue("Chrome's 404 fancy-page sucks." not in resp.body)
+        self.assertNotIn("Chrome's 404 fancy-page sucks.", resp.body)
 
     def test_container3bindexhtml(self):
         resp = Request.blank('/v1/a/c3b/').get_response(self.test_staticweb)
@@ -603,7 +616,7 @@ class TestStaticWeb(unittest.TestCase):
             '/v1/a/c4/subdir/').get_response(self.test_staticweb)
         self.assertEqual(resp.status_int, 200)
         self.assertTrue('Listing of /v1/a/c4/subdir/' in resp.body)
-        self.assertTrue('</style>' not in resp.body)
+        self.assertNotIn('</style>', resp.body)
         self.assertTrue('<link' in resp.body)
         self.assertTrue('href="../listing.css"' in resp.body)
         self.assertEqual(resp.headers['content-type'],
@@ -627,7 +640,7 @@ class TestStaticWeb(unittest.TestCase):
         resp = Request.blank(
             '/v1/a/c5/unknown').get_response(self.test_staticweb)
         self.assertEqual(resp.status_int, 404)
-        self.assertTrue("Chrome's 404 fancy-page sucks." not in resp.body)
+        self.assertNotIn("Chrome's 404 fancy-page sucks.", resp.body)
 
     def test_container6subdir(self):
         resp = Request.blank(
@@ -663,9 +676,37 @@ class TestStaticWeb(unittest.TestCase):
         self.assertIn(label, resp.body)
 
     def test_container7listing(self):
+        # container7 has web-listings = f, web-error=error.html
         resp = Request.blank('/v1/a/c7/').get_response(self.test_staticweb)
         self.assertEqual(resp.status_int, 404)
-        self.assertTrue('Web Listing Disabled' in resp.body)
+        self.assertIn("Web Listing Disabled", resp.body)
+
+        # expect 301 if auth'd but no trailing '/'
+        resp = Request.blank('/v1/a/c7').get_response(self.test_staticweb)
+        self.assertEqual(resp.status_int, 301)
+
+        # expect default 401 if request is not auth'd and no trailing '/'
+        test_staticweb = FakeAuthFilter(
+            staticweb.filter_factory({})(self.app), deny_listing=True,
+            deny_objects=True)
+        resp = Request.blank('/v1/a/c7').get_response(test_staticweb)
+        self.assertEqual(resp.status_int, 401)
+        self.assertNotIn("Hey, you're not authorized to see this!", resp.body)
+
+        # expect custom 401 if request is not auth'd for listing
+        test_staticweb = FakeAuthFilter(
+            staticweb.filter_factory({})(self.app), deny_listing=True)
+        resp = Request.blank('/v1/a/c7/').get_response(test_staticweb)
+        self.assertEqual(resp.status_int, 401)
+        self.assertIn("Hey, you're not authorized to see this!", resp.body)
+
+        # expect default 401 if request is not auth'd for listing or object GET
+        test_staticweb = FakeAuthFilter(
+            staticweb.filter_factory({})(self.app), deny_listing=True,
+            deny_objects=True)
+        resp = Request.blank('/v1/a/c7/').get_response(test_staticweb)
+        self.assertEqual(resp.status_int, 401)
+        self.assertNotIn("Hey, you're not authorized to see this!", resp.body)
 
     def test_container8listingcss(self):
         resp = Request.blank(
@@ -755,7 +796,7 @@ class TestStaticWeb(unittest.TestCase):
         resp = Request.blank('/v1/a/c7/').get_response(
             self.test_staticweb)
         self.assertEqual(resp.status_int, 404)
-        self.assertTrue('listing.css' not in resp.body)
+        self.assertNotIn('listing.css', resp.body)
         self.assertTrue('<style' in resp.body)
 
     def test_subrequest_once_if_possible(self):
@@ -802,5 +843,56 @@ class TestStaticWeb(unittest.TestCase):
             self.assertEqual(resp.status_int, 503)  # sanity
 
 
+class TestStaticWebUrlBase(unittest.TestCase):
+
+    def setUp(self):
+        self.app = FakeApp()
+        self._orig_get_container_info = staticweb.get_container_info
+        staticweb.get_container_info = mock_get_container_info
+
+    def tearDown(self):
+        staticweb.get_container_info = self._orig_get_container_info
+
+    def test_container3subdirz_scheme(self):
+        path = '/v1/a/c3/subdirz'
+        scheme = 'https'
+        test_staticweb = FakeAuthFilter(
+            staticweb.filter_factory({'url_base': 'https://'})(self.app))
+        resp = Request.blank(path).get_response(test_staticweb)
+        self.assertEqual(resp.status_int, 301)
+        parsed = urlparse(resp.location)
+        self.assertEqual(parsed.scheme, scheme)
+        # We omit comparing netloc here, because swob is free to add port.
+        self.assertEqual(parsed.path, path + '/')
+
+    def test_container3subdirz_host(self):
+        path = '/v1/a/c3/subdirz'
+        netloc = 'example.com'
+        test_staticweb = FakeAuthFilter(
+            staticweb.filter_factory({
+                'url_base': '//%s' % (netloc,)})(self.app))
+        resp = Request.blank(path).get_response(test_staticweb)
+        self.assertEqual(resp.status_int, 301)
+        parsed = urlparse(resp.location)
+        # We compare scheme with the default. This may change, but unlikely.
+        self.assertEqual(parsed.scheme, 'http')
+        self.assertEqual(parsed.netloc, netloc)
+        self.assertEqual(parsed.path, path + '/')
+
+    def test_container3subdirz_both(self):
+        path = '/v1/a/c3/subdirz'
+        scheme = 'http'
+        netloc = 'example.com'
+        test_staticweb = FakeAuthFilter(
+            staticweb.filter_factory({
+                'url_base': 'http://example.com'})(self.app))
+        resp = Request.blank(path).get_response(test_staticweb)
+        self.assertEqual(resp.status_int, 301)
+        parsed = urlparse(resp.location)
+        self.assertEqual(parsed.scheme, scheme)
+        self.assertEqual(parsed.netloc, netloc)
+        self.assertEqual(parsed.path, path + '/')
+
+
 if __name__ == '__main__':
     unittest.main()
