@@ -24,6 +24,8 @@ import errno
 from collections import defaultdict
 from time import sleep, time
 
+from six.moves import reload_module
+
 from swift.common import manager
 from swift.common.exceptions import InvalidPidFileException
 
@@ -283,7 +285,7 @@ class TestManagerModule(unittest.TestCase):
 class TestServer(unittest.TestCase):
 
     def tearDown(self):
-        reload(manager)
+        reload_module(manager)
 
     def join_swift_dir(self, path):
         return os.path.join(manager.SWIFT_DIR, path)
@@ -603,19 +605,19 @@ class TestServer(unittest.TestCase):
             manager.RUN_DIR = t
             server = manager.Server('proxy', run_dir=t)
             # test get one file
-            iter = server.iter_pid_files()
-            pid_file, pid = next(iter)
+            iterator = server.iter_pid_files()
+            pid_file, pid = next(iterator)
             self.assertEqual(pid_file, self.join_run_dir('proxy-server.pid'))
             self.assertEqual(pid, 1)
             # ... and only one file
-            self.assertRaises(StopIteration, iter.next)
+            self.assertRaises(StopIteration, next, iterator)
             # test invalid value in pid file
             server = manager.Server('auth', run_dir=t)
             pid_file, pid = next(server.iter_pid_files())
             self.assertIsNone(pid)
             # test object-server doesn't steal pids from object-replicator
             server = manager.Server('object', run_dir=t)
-            self.assertRaises(StopIteration, server.iter_pid_files().next)
+            self.assertRaises(StopIteration, next, server.iter_pid_files())
             # test multi-pid iter
             server = manager.Server('object-replicator', run_dir=t)
             real_map = {
@@ -739,8 +741,8 @@ class TestServer(unittest.TestCase):
                     manager.os = MockOs([2])
                     # test pid not running
                     pids = server.signal_pids(signal.SIG_DFL)
-                    self.assertTrue(1 not in pids)
-                    self.assertTrue(1 not in manager.os.pid_sigs)
+                    self.assertNotIn(1, pids)
+                    self.assertNotIn(1, manager.os.pid_sigs)
                     # test remove stale pid file
                     self.assertFalse(os.path.exists(
                         self.join_run_dir('proxy-server.pid')))
@@ -816,8 +818,8 @@ class TestServer(unittest.TestCase):
             running_pids = server.get_running_pids()
             self.assertEqual(len(running_pids), 1)
             self.assertTrue(1 in running_pids)
-            self.assertTrue(2 not in running_pids)
-            self.assertTrue(3 not in running_pids)
+            self.assertNotIn(2, running_pids)
+            self.assertNotIn(3, running_pids)
             # test persistent running pid files
             self.assertTrue(os.path.exists(
                 os.path.join(manager.RUN_DIR, 'test-server1.pid')))
@@ -855,7 +857,7 @@ class TestServer(unittest.TestCase):
             self.assertTrue(1 in running_pids)
             # no other pids returned
             for n in (2, 3, 4):
-                self.assertTrue(n not in running_pids)
+                self.assertNotIn(n, running_pids)
             # assert stale pids for other servers ignored
             manager.os = MockOs([1])  # only thing-doer is running
             running_pids = server.get_running_pids()
@@ -920,7 +922,7 @@ class TestServer(unittest.TestCase):
                 self.assertEqual(manager.os.pid_sigs[pid],
                                  [signal.SIGTERM])
             # and the other pid is of course not signaled
-            self.assertTrue(1 not in manager.os.pid_sigs)
+            self.assertNotIn(1, manager.os.pid_sigs)
 
     def test_status(self):
         conf_files = (
@@ -1127,9 +1129,13 @@ class TestServer(unittest.TestCase):
                     self.assertEqual(len(server.procs), 3)
                     proc = server.procs[2]
                     # assert stdout is /dev/null
-                    self.assertTrue(isinstance(proc.stdout, file))
+                    with open('/dev/null', 'wb+') as fp:
+                        self.assertTrue(isinstance(proc.stdout, type(fp)))
                     self.assertEqual(proc.stdout.name, os.devnull)
-                    self.assertEqual(proc.stdout.mode, 'w+b')
+                    self.assertIn('b', proc.stdout.mode)
+                    self.assertTrue(any(x in proc.stdout.mode for x in 'aw+'),
+                                    'mode must be writable, not %r' %
+                                    proc.stdout.mode)
                     self.assertEqual(proc.stderr, proc.stdout)
                     # test not daemon over-rides wait
                     server.spawn(conf4, wait=False, daemon=False, once=True)
@@ -1243,7 +1249,7 @@ class TestServer(unittest.TestCase):
                     self.assertTrue('mock process started' in output)
                     self.assertTrue('setup complete' in output)
                     # make sure we don't get prints after stdout was closed
-                    self.assertTrue('mock process finished' not in output)
+                    self.assertNotIn('mock process finished', output)
                     # test process which fails to start
                     with MockProcess(fail_to_start=True) as proc:
                         server.procs = [proc]
@@ -1384,7 +1390,7 @@ class TestServer(unittest.TestCase):
                             self.assertEqual(mock_spawn.kwargs, [expected])
                             output = pop_stream(f)
                             self.assertTrue('Starting' in output)
-                            self.assertTrue('once' not in output)
+                            self.assertNotIn('once', output)
                         # test multi-server kwarg once
                         server = manager.Server('object-replicator')
                         with temptree([], []) as proc_dir:
@@ -2039,7 +2045,6 @@ class TestManager(unittest.TestCase):
             manager.watch_server_pids = _orig_watch_server_pids
             manager.kill_group = _orig_kill_group
 
-    # TODO(clayg): more tests
     def test_shutdown(self):
         m = manager.Manager(['test'])
         m.stop_was_called = False
