@@ -12,7 +12,7 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
+import collections
 import unittest
 import os
 import mock
@@ -26,10 +26,10 @@ from collections import defaultdict
 from errno import ENOENT, ENOTEMPTY, ENOTDIR
 
 from eventlet.green import subprocess
-from eventlet import Timeout, tpool
+from eventlet import Timeout
 
 from test.unit import (debug_logger, patch_policies, make_timestamp_iter,
-                       mocked_http_conn)
+                       mocked_http_conn, FakeLogger)
 from swift.common import utils
 from swift.common.utils import (hash_path, mkdirs, normalize_timestamp,
                                 storage_directory)
@@ -140,19 +140,20 @@ def _create_test_rings(path, devs=None):
     ]
     intended_devs = devs or [
         {'id': 0, 'device': 'sda', 'zone': 0,
-         'region': 1, 'ip': '127.0.0.0', 'port': 6000},
+         'region': 1, 'ip': '127.0.0.0', 'port': 6200},
         {'id': 1, 'device': 'sda', 'zone': 1,
-         'region': 2, 'ip': '127.0.0.1', 'port': 6000},
+         'region': 2, 'ip': '127.0.0.1', 'port': 6200},
         {'id': 2, 'device': 'sda', 'zone': 2,
-         'region': 3, 'ip': '127.0.0.2', 'port': 6000},
+         'region': 3, 'ip': '127.0.0.2', 'port': 6200},
         {'id': 3, 'device': 'sda', 'zone': 4,
-         'region': 2, 'ip': '127.0.0.3', 'port': 6000},
+         'region': 2, 'ip': '127.0.0.3', 'port': 6200},
         {'id': 4, 'device': 'sda', 'zone': 5,
-         'region': 1, 'ip': '127.0.0.4', 'port': 6000},
+         'region': 1, 'ip': '127.0.0.4', 'port': 6200,
+         'replication_ip': '127.0.1.4'},
         {'id': 5, 'device': 'sda', 'zone': 6,
-         'region': 3, 'ip': 'fe80::202:b3ff:fe1e:8329', 'port': 6000},
+         'region': 3, 'ip': 'fe80::202:b3ff:fe1e:8329', 'port': 6200},
         {'id': 6, 'device': 'sda', 'zone': 7, 'region': 1,
-         'ip': '2001:0db8:85a3:0000:0000:8a2e:0370:7334', 'port': 6000},
+         'ip': '2001:0db8:85a3:0000:0000:8a2e:0370:7334', 'port': 6200},
     ]
     intended_part_shift = 30
     with closing(GzipFile(testgz, 'wb')) as f:
@@ -195,7 +196,7 @@ class TestObjectReplicator(unittest.TestCase):
         _create_test_rings(self.testdir)
         self.logger = debug_logger('test-replicator')
         self.conf = dict(
-            bind_ip=_ips()[0], bind_port=6000,
+            bind_ip=_ips()[0], bind_port=6200,
             swift_dir=self.testdir, devices=self.devices, mount_check='false',
             timeout='300', stats_interval='1', sync_method='rsync')
         self._create_replicator()
@@ -274,10 +275,10 @@ class TestObjectReplicator(unittest.TestCase):
                                                         logger=self.logger)
         replicator.run_once()
         expected = [
-            "Can't find itself 1.1.1.1 with port 6000 "
-            "in ring file, not replicating",
-            "Can't find itself 1.1.1.1 with port 6000 "
-            "in ring file, not replicating",
+            "Can't find itself in policy with index 0 with ips 1.1.1.1 and"
+            " with port 6200 in ring file, not replicating",
+            "Can't find itself in policy with index 1 with ips 1.1.1.1 and"
+            " with port 6200 in ring file, not replicating",
         ]
         self.assertEqual(expected, self.logger.get_lines_for_level('error'))
 
@@ -313,10 +314,10 @@ class TestObjectReplicator(unittest.TestCase):
                 (0, '', ['rsync', whole_path_from, rsync_mods]))
         start = replicator.replication_cycle
         self.assertGreaterEqual(start, 0)
-        self.assertLess(start, 9)
+        self.assertLessEqual(start, 9)
         with _mock_process(process_arg_checker):
             replicator.run_once()
-        self.assertEqual(start + 1, replicator.replication_cycle)
+        self.assertEqual((start + 1) % 10, replicator.replication_cycle)
         self.assertFalse(process_errors)
         self.assertFalse(self.logger.get_lines_for_level('error'))
         object_replicator.http_connect = was_connector
@@ -437,16 +438,16 @@ class TestObjectReplicator(unittest.TestCase):
         devs = [
             {'id': 0, 'device': 'sda', 'zone': 0,
              'region': 1, 'ip': '1.1.1.1', 'port': 1111,
-             'replication_ip': '127.0.0.0', 'replication_port': 6000},
+             'replication_ip': '127.0.0.0', 'replication_port': 6200},
             {'id': 1, 'device': 'sdb', 'zone': 1,
              'region': 1, 'ip': '1.1.1.1', 'port': 1111,
-             'replication_ip': '127.0.0.0', 'replication_port': 6000},
+             'replication_ip': '127.0.0.0', 'replication_port': 6200},
             {'id': 2, 'device': 'sdc', 'zone': 2,
              'region': 1, 'ip': '1.1.1.1', 'port': 1111,
-             'replication_ip': '127.0.0.1', 'replication_port': 6000},
+             'replication_ip': '127.0.0.1', 'replication_port': 6200},
             {'id': 3, 'device': 'sdd', 'zone': 3,
              'region': 1, 'ip': '1.1.1.1', 'port': 1111,
-             'replication_ip': '127.0.0.1', 'replication_port': 6000},
+             'replication_ip': '127.0.0.1', 'replication_port': 6200},
         ]
         objects_sdb, objects_1_sdb, _, _ = \
             self._write_disk_data('sdb', with_json=True)
@@ -465,17 +466,17 @@ class TestObjectReplicator(unittest.TestCase):
             # Two disks on same IP/port
             {'id': 0, 'device': 'sda', 'zone': 0,
              'region': 1, 'ip': '1.1.1.1', 'port': 1111,
-             'replication_ip': '127.0.0.0', 'replication_port': 6000},
+             'replication_ip': '127.0.0.0', 'replication_port': 6200},
             {'id': 1, 'device': 'sdb', 'zone': 1,
              'region': 1, 'ip': '1.1.1.1', 'port': 1111,
-             'replication_ip': '127.0.0.0', 'replication_port': 6000},
+             'replication_ip': '127.0.0.0', 'replication_port': 6200},
             # Two disks on same server, different ports
             {'id': 2, 'device': 'sdc', 'zone': 2,
              'region': 2, 'ip': '1.1.1.2', 'port': 1112,
-             'replication_ip': '127.0.0.1', 'replication_port': 6000},
+             'replication_ip': '127.0.0.1', 'replication_port': 6200},
             {'id': 3, 'device': 'sdd', 'zone': 4,
              'region': 2, 'ip': '1.1.1.2', 'port': 1112,
-             'replication_ip': '127.0.0.1', 'replication_port': 6001},
+             'replication_ip': '127.0.0.1', 'replication_port': 6201},
         ]
         objects_sdb, objects_1_sdb, _, _ = self._write_disk_data('sdb')
         objects_sdc, objects_1_sdc, _, _ = self._write_disk_data('sdc')
@@ -564,22 +565,22 @@ class TestObjectReplicator(unittest.TestCase):
     def test_collect_jobs_multi_disk_diff_ports_normal(self, mock_shuffle):
         # Normally (servers_per_port=0), replication_ip AND replication_port
         # are used to determine local ring device entries.  Here we show that
-        # with bind_ip='127.0.0.1', bind_port=6000, only "sdc" is local.
+        # with bind_ip='127.0.0.1', bind_port=6200, only "sdc" is local.
         devs = [
             # Two disks on same IP/port
             {'id': 0, 'device': 'sda', 'zone': 0,
              'region': 1, 'ip': '1.1.1.1', 'port': 1111,
-             'replication_ip': '127.0.0.0', 'replication_port': 6000},
+             'replication_ip': '127.0.0.0', 'replication_port': 6200},
             {'id': 1, 'device': 'sdb', 'zone': 1,
              'region': 1, 'ip': '1.1.1.1', 'port': 1111,
-             'replication_ip': '127.0.0.0', 'replication_port': 6000},
+             'replication_ip': '127.0.0.0', 'replication_port': 6200},
             # Two disks on same server, different ports
             {'id': 2, 'device': 'sdc', 'zone': 2,
              'region': 2, 'ip': '1.1.1.2', 'port': 1112,
-             'replication_ip': '127.0.0.1', 'replication_port': 6000},
+             'replication_ip': '127.0.0.1', 'replication_port': 6200},
             {'id': 3, 'device': 'sdd', 'zone': 4,
              'region': 2, 'ip': '1.1.1.2', 'port': 1112,
-             'replication_ip': '127.0.0.1', 'replication_port': 6001},
+             'replication_ip': '127.0.0.1', 'replication_port': 6201},
         ]
         objects_sdb, objects_1_sdb, _, _ = self._write_disk_data('sdb')
         objects_sdc, objects_1_sdc, _, _ = self._write_disk_data('sdc')
@@ -643,23 +644,23 @@ class TestObjectReplicator(unittest.TestCase):
     def test_collect_jobs_multi_disk_servers_per_port(self, mock_shuffle):
         # Normally (servers_per_port=0), replication_ip AND replication_port
         # are used to determine local ring device entries.  Here we show that
-        # with servers_per_port > 0 and bind_ip='127.0.0.1', bind_port=6000,
+        # with servers_per_port > 0 and bind_ip='127.0.0.1', bind_port=6200,
         # then both "sdc" and "sdd" are local.
         devs = [
             # Two disks on same IP/port
             {'id': 0, 'device': 'sda', 'zone': 0,
              'region': 1, 'ip': '1.1.1.1', 'port': 1111,
-             'replication_ip': '127.0.0.0', 'replication_port': 6000},
+             'replication_ip': '127.0.0.0', 'replication_port': 6200},
             {'id': 1, 'device': 'sdb', 'zone': 1,
              'region': 1, 'ip': '1.1.1.1', 'port': 1111,
-             'replication_ip': '127.0.0.0', 'replication_port': 6000},
+             'replication_ip': '127.0.0.0', 'replication_port': 6200},
             # Two disks on same server, different ports
             {'id': 2, 'device': 'sdc', 'zone': 2,
              'region': 2, 'ip': '1.1.1.2', 'port': 1112,
-             'replication_ip': '127.0.0.1', 'replication_port': 6000},
+             'replication_ip': '127.0.0.1', 'replication_port': 6200},
             {'id': 3, 'device': 'sdd', 'zone': 4,
              'region': 2, 'ip': '1.1.1.2', 'port': 1112,
-             'replication_ip': '127.0.0.1', 'replication_port': 6001},
+             'replication_ip': '127.0.0.1', 'replication_port': 6201},
         ]
         objects_sdb, objects_1_sdb, _, _ = self._write_disk_data('sdb')
         objects_sdc, objects_1_sdc, _, _ = self._write_disk_data('sdc')
@@ -952,6 +953,49 @@ class TestObjectReplicator(unittest.TestCase):
         jobs = self.replicator.collect_jobs()
         self.assertEqual(len(jobs), 0)
 
+    def test_replicator_skips_rsync_temp_files(self):
+        # the empty pre-setup dirs aren't that useful to us
+        device_path = os.path.join(self.devices, 'sda')
+        rmtree(device_path, ignore_errors=1)
+        os.mkdir(device_path)
+        # create a real data file to trigger rsync
+        df = self.df_mgr.get_diskfile('sda', '0', 'a', 'c', 'o',
+                                      policy=POLICIES.legacy)
+        ts = next(self.ts)
+        with df.create() as w:
+            w.write('asdf')
+            w.put({'X-Timestamp': ts.internal})
+            w.commit(ts)
+        # pre-flight and post sync request for both other primaries
+        expected_replicate_requests = 4
+        process_arg_checker = [
+            # (return_code, stdout, <each in capture rsync args>)
+            (0, '', []),
+            (0, '', []),
+        ]
+        stub_body = pickle.dumps({})
+        with _mock_process(process_arg_checker) as rsync_log, \
+            mock.patch('swift.obj.replicator.whataremyips',
+                       side_effect=_ips), \
+                mocked_http_conn(*[200] * expected_replicate_requests,
+                                 body=stub_body) as conn_log:
+            self.replicator.replicate()
+        self.assertEqual(['REPLICATE'] * expected_replicate_requests,
+                         [r['method'] for r in conn_log.requests])
+        # expect one rsync to each other primary node
+        self.assertEqual(2, len(rsync_log))
+        expected = '--exclude=.*.[0-9a-zA-Z][0-9a-zA-Z][0-9a-zA-Z]' \
+            '[0-9a-zA-Z][0-9a-zA-Z][0-9a-zA-Z]'
+        for subprocess_info in rsync_log:
+            rsync_args = subprocess_info['rsync_args']
+            for arg in rsync_args:
+                if arg.startswith('--exclude'):
+                    self.assertEqual(arg, expected)
+                    break
+            else:
+                self.fail('Did not find --exclude argument in %r' %
+                          rsync_args)
+
     def test_replicator_removes_zbf(self):
         # After running xfs_repair, a partition directory could become a
         # zero-byte file. If this happens, the replicator should clean it
@@ -1054,19 +1098,19 @@ class TestObjectReplicator(unittest.TestCase):
     def test_delete_partition_ssync_single_region(self):
         devs = [
             {'id': 0, 'device': 'sda', 'zone': 0,
-             'region': 1, 'ip': '127.0.0.0', 'port': 6000},
+             'region': 1, 'ip': '127.0.0.0', 'port': 6200},
             {'id': 1, 'device': 'sda', 'zone': 1,
-             'region': 1, 'ip': '127.0.0.1', 'port': 6000},
+             'region': 1, 'ip': '127.0.0.1', 'port': 6200},
             {'id': 2, 'device': 'sda', 'zone': 2,
-             'region': 1, 'ip': '127.0.0.2', 'port': 6000},
+             'region': 1, 'ip': '127.0.0.2', 'port': 6200},
             {'id': 3, 'device': 'sda', 'zone': 4,
-             'region': 1, 'ip': '127.0.0.3', 'port': 6000},
+             'region': 1, 'ip': '127.0.0.3', 'port': 6200},
             {'id': 4, 'device': 'sda', 'zone': 5,
-             'region': 1, 'ip': '127.0.0.4', 'port': 6000},
+             'region': 1, 'ip': '127.0.0.4', 'port': 6200},
             {'id': 5, 'device': 'sda', 'zone': 6,
-             'region': 1, 'ip': 'fe80::202:b3ff:fe1e:8329', 'port': 6000},
+             'region': 1, 'ip': 'fe80::202:b3ff:fe1e:8329', 'port': 6200},
             {'id': 6, 'device': 'sda', 'zone': 7, 'region': 1,
-             'ip': '2001:0db8:85a3:0000:0000:8a2e:0370:7334', 'port': 6000},
+             'ip': '2001:0db8:85a3:0000:0000:8a2e:0370:7334', 'port': 6200},
         ]
         _create_test_rings(self.testdir, devs=devs)
         self.conf['sync_method'] = 'ssync'
@@ -1579,68 +1623,80 @@ class TestObjectReplicator(unittest.TestCase):
             object_replicator.http_connect = was_connector
 
     def test_run_once_recover_from_timeout(self):
+        # verify that replicator will pass over all policies' partitions even
+        # if a timeout occurs while replicating one partition to one node.
+        timeouts = [Timeout()]
+
+        def fake_get_hashes(df_mgr, part_path, **kwargs):
+            self.get_hash_count += 1
+            # Simulate a REPLICATE timeout by raising Timeout for second call
+            # to get_hashes (with recalculate suffixes) for a specific
+            # partition
+            if (timeouts and '/objects/' in part_path and
+                    part_path.endswith('0') and 'recalculate' in kwargs):
+                raise timeouts.pop(0)
+            return 1, {'abc': 'def'}
+
+        # map partition_path -> [nodes]
+        sync_paths = collections.defaultdict(list)
+
+        def fake_sync(node, job, suffixes, *args, **kwargs):
+            sync_paths[job['path']].append(node)
+            return True, {}
+
         conf = dict(swift_dir=self.testdir, devices=self.devices,
-                    bind_ips=_ips()[0],
+                    bind_ip=_ips()[0],  # local dev has id=0
                     mount_check='false', timeout='300', stats_interval='1')
-        replicator = object_replicator.ObjectReplicator(conf)
-        was_connector = object_replicator.http_connect
-        was_get_hashes = object_replicator.DiskFileManager._get_hashes
-        was_execute = tpool.execute
-        self.get_hash_count = 0
-        try:
+        with mock.patch('swift.obj.diskfile.DiskFileManager._get_hashes',
+                        fake_get_hashes):
+            with mock.patch('swift.obj.replicator.http_connect',
+                            mock_http_connect(200)):
+                with mock.patch('swift.obj.replicator.dump_recon_cache'):
+                    replicator = object_replicator.ObjectReplicator(
+                        conf, logger=FakeLogger())
 
-            def fake_get_hashes(*args, **kwargs):
-                self.get_hash_count += 1
-                if self.get_hash_count == 3:
-                    # raise timeout on last call to get hashes
-                    raise Timeout()
-                return 2, {'abc': 'def'}
+                    self.get_hash_count = 0
+                    with mock.patch.object(replicator, 'sync', fake_sync):
+                        replicator.run_once()
 
-            def fake_exc(tester, *args, **kwargs):
-                if 'Error syncing partition timeout' in args[0]:
-                    tester.i_failed = True
+        log_lines = replicator.logger.get_lines_for_level('error')
+        self.assertIn("Error syncing with node:", log_lines[0])
+        self.assertFalse(log_lines[1:])
+        # setup creates 4 partitions; partition 1 does not map to local dev id
+        # 0 so will be handled by update_delete(); partitions 0, 2, 3 are
+        # handled by update() for each of two policies, so expect 6 paths to be
+        # sync'd
+        self.assertEqual(6, len(sync_paths))
+        # partition 3 has 2 nodes in remote region, only first node is sync'd.
+        # partition 0 in policy 0 has fake_get_hashes timeout before first
+        # sync, so only second node is sync'd.
+        # other partitions are sync'd to 2 nodes in same region.
+        expected_node_count = {  # map path_end -> expected sync node count
+            '/objects/0': 1,
+            '/objects/1': 2,
+            '/objects/2': 2,
+            '/objects/3': 1,
+            '/objects-1/0': 2,
+            '/objects-1/1': 2,
+            '/objects-1/2': 2,
+            '/objects-1/3': 1
+        }
+        for path, nodes in sync_paths.items():
+            path_end = path[path.index('/objects'):]
+            self.assertEqual(expected_node_count[path_end], len(nodes),
+                             'Expected %s but got %s for path %s' %
+                             (expected_node_count[path_end], len(nodes), path))
+        # partitions 0 and 2 attempt 3 calls each per policy to get_hashes = 12
+        # partitions 3 attempts 2 calls per policy to get_hashes = 4
+        # partitions 1 dosn't get_hashes because of update_deleted
+        self.assertEqual(16, self.get_hash_count)
 
-            self.i_failed = False
-            object_replicator.http_connect = mock_http_connect(200)
-            object_replicator.DiskFileManager._get_hashes = fake_get_hashes
-            replicator.logger.exception = \
-                lambda *args, **kwargs: fake_exc(self, *args, **kwargs)
-            # Write some files into '1' and run replicate- they should be moved
-            # to the other partitions and then node should get deleted.
-            cur_part = '1'
-            df = self.df_mgr.get_diskfile('sda', cur_part, 'a', 'c', 'o',
-                                          policy=POLICIES.legacy)
-            mkdirs(df._datadir)
-            f = open(os.path.join(df._datadir,
-                                  normalize_timestamp(time.time()) + '.data'),
-                     'wb')
-            f.write('1234567890')
-            f.close()
-            ohash = hash_path('a', 'c', 'o')
-            data_dir = ohash[-3:]
-            whole_path_from = os.path.join(self.objects, cur_part, data_dir)
-            process_arg_checker = []
-            ring = replicator.load_object_ring(POLICIES[0])
-            nodes = [node for node in
-                     ring.get_part_nodes(int(cur_part))
-                     if node['ip'] not in _ips()]
-
-            for node in nodes:
-                rsync_mod = '%s::object/sda/objects/%s' % (node['ip'],
-                                                           cur_part)
-                process_arg_checker.append(
-                    (0, '', ['rsync', whole_path_from, rsync_mod]))
-            self.assertTrue(os.access(os.path.join(self.objects,
-                                                   '1', data_dir, ohash),
-                                      os.F_OK))
-            with _mock_process(process_arg_checker):
-                replicator.run_once()
-            self.assertFalse(process_errors)
-            self.assertFalse(self.i_failed)
-        finally:
-            object_replicator.http_connect = was_connector
-            object_replicator.DiskFileManager._get_hashes = was_get_hashes
-            tpool.execute = was_execute
+        # attempt to 16 times but succeeded only 15 times due to Timeout
+        suffix_hashes = sum(
+            count for (metric, count), _junk in
+            replicator.logger.log_dict['update_stats']
+            if metric == 'suffix.hashes')
+        self.assertEqual(15, suffix_hashes)
 
     def test_run(self):
         with _mock_process([(0, '')] * 100):
@@ -1674,7 +1730,6 @@ class TestObjectReplicator(unittest.TestCase):
 
         self.headers = {'Content-Length': '0',
                         'user-agent': 'object-replicator %s' % os.getpid()}
-        self.replicator.logger = mock_logger = mock.MagicMock()
         mock_tpool_reraise.return_value = (0, {})
 
         all_jobs = self.replicator.collect_jobs()
@@ -1685,8 +1740,8 @@ class TestObjectReplicator(unittest.TestCase):
         # Check incorrect http_connect with status 507 and
         # count of attempts and call args
         resp.status = 507
-        error = '%(ip)s/%(device)s responded as unmounted'
-        expect = 'Error syncing partition'
+        error = '%(replication_ip)s/%(device)s responded as unmounted'
+        expect = 'Error syncing partition: '
         expected_listdir_calls = [
             mock.call(int(job['partition']),
                       self.replicator.replication_cycle)
@@ -1694,9 +1749,9 @@ class TestObjectReplicator(unittest.TestCase):
         do_listdir_results = [False, False, True, False, True, False]
         mock_do_listdir.side_effect = do_listdir_results
         expected_tpool_calls = [
-            mock.call(self.replicator._diskfile_mgr._get_hashes, job['path'],
-                      do_listdir=do_listdir,
-                      reclaim_age=self.replicator.reclaim_age)
+            mock.call(self.replicator._df_router[job['policy']]._get_hashes,
+                      job['path'],
+                      do_listdir=do_listdir)
             for job, do_listdir in zip(jobs, do_listdir_results)
         ]
         for job in jobs:
@@ -1704,8 +1759,18 @@ class TestObjectReplicator(unittest.TestCase):
             ring = job['policy'].object_ring
             self.headers['X-Backend-Storage-Policy-Index'] = int(job['policy'])
             self.replicator.update(job)
-            self.assertTrue(error in mock_logger.error.call_args[0][0])
-            self.assertTrue(expect in mock_logger.exception.call_args[0][0])
+            error_lines = self.logger.get_lines_for_level('error')
+            expected = []
+            # ... first the primaries
+            for node in job['nodes']:
+                expected.append(error % node)
+            # ... then it will get handoffs
+            for node in job['policy'].object_ring.get_more_nodes(
+                    int(job['partition'])):
+                expected.append(error % node)
+            # ... and finally it will exception out
+            expected.append(expect)
+            self.assertEqual(expected, error_lines)
             self.assertEqual(len(self.replicator.partition_times), 1)
             self.assertEqual(mock_http.call_count, len(ring._devs) - 1)
             reqs = []
@@ -1717,8 +1782,7 @@ class TestObjectReplicator(unittest.TestCase):
                 self.assertEqual(self.replicator.suffix_hash, 0)
             mock_http.assert_has_calls(reqs, any_order=True)
             mock_http.reset_mock()
-            mock_logger.reset_mock()
-
+            self.logger.clear()
         mock_do_listdir.assert_has_calls(expected_listdir_calls)
         mock_tpool_reraise.assert_has_calls(expected_tpool_calls)
         mock_do_listdir.side_effect = None
@@ -1729,20 +1793,27 @@ class TestObjectReplicator(unittest.TestCase):
         for job in jobs:
             set_default(self)
             self.replicator.update(job)
-            self.assertTrue(error in mock_logger.error.call_args[0][0])
+            # ... only the primaries
+            expected = [error % {'resp': 400, 'ip': node['replication_ip']}
+                        for node in job['nodes']]
+            self.assertEqual(expected,
+                             self.logger.get_lines_for_level('error'))
             self.assertEqual(len(self.replicator.partition_times), 1)
-            mock_logger.reset_mock()
+            self.logger.clear()
 
         # Check successful http_connection and exception with
         # incorrect pickle.loads(resp.read())
         resp.status = 200
-        expect = 'Error syncing with node:'
+        expect = 'Error syncing with node: %r: '
         for job in jobs:
             set_default(self)
             self.replicator.update(job)
-            self.assertTrue(expect in mock_logger.exception.call_args[0][0])
+            # ... only the primaries
+            expected = [expect % node for node in job['nodes']]
+            error_lines = self.logger.get_lines_for_level('error')
+            self.assertEqual(expected, error_lines)
             self.assertEqual(len(self.replicator.partition_times), 1)
-            mock_logger.reset_mock()
+            self.logger.clear()
 
         # Check successful http_connection and correct
         # pickle.loads(resp.read()) for non local node
@@ -1756,13 +1827,12 @@ class TestObjectReplicator(unittest.TestCase):
                 local_job = job.copy()
                 continue
             self.replicator.update(job)
-            self.assertEqual(mock_logger.exception.call_count, 0)
-            self.assertEqual(mock_logger.error.call_count, 0)
+            self.assertEqual([], self.logger.get_lines_for_level('error'))
             self.assertEqual(len(self.replicator.partition_times), 1)
             self.assertEqual(self.replicator.suffix_hash, 0)
             self.assertEqual(self.replicator.suffix_sync, 0)
             self.assertEqual(self.replicator.suffix_count, 0)
-            mock_logger.reset_mock()
+            self.logger.clear()
 
         # Check successful http_connect and sync for local node
         mock_tpool_reraise.return_value = (1, {'a83': 'ba47fd314242ec8c'
@@ -1803,7 +1873,7 @@ class TestObjectReplicator(unittest.TestCase):
         self.assertEqual(self.replicator.suffix_count, 1)
 
         mock_http.reset_mock()
-        mock_logger.reset_mock()
+        self.logger.clear()
 
         # test for replication params on policy 0 only
         repl_job = local_job.copy()
@@ -1832,37 +1902,36 @@ class TestObjectReplicator(unittest.TestCase):
         jobs = self.replicator.collect_jobs()
         _m_rsync = mock.Mock(return_value=0)
         _m_os_path_exists = mock.Mock(return_value=True)
-        with mock.patch.object(self.replicator, '_rsync', _m_rsync):
-            with mock.patch('os.path.exists', _m_os_path_exists):
-                for job in jobs:
-                    self.assertTrue('region' in job)
-                    for node in job['nodes']:
-                        for rsync_compress in (True, False):
-                            self.replicator.rsync_compress = rsync_compress
-                            ret = \
-                                self.replicator.sync(node, job,
-                                                     ['fake_suffix'])
-                            self.assertTrue(ret)
-                            if node['region'] != job['region']:
-                                if rsync_compress:
-                                    # --compress arg should be passed to rsync
-                                    # binary only when rsync_compress option is
-                                    # enabled AND destination node is in a
-                                    # different region
-                                    self.assertTrue('--compress' in
-                                                    _m_rsync.call_args[0][0])
-                                else:
-                                    self.assertFalse('--compress' in
-                                                     _m_rsync.call_args[0][0])
+        with mock.patch.object(self.replicator, '_rsync', _m_rsync), \
+                mock.patch('os.path.exists', _m_os_path_exists):
+            for job in jobs:
+                self.assertTrue('region' in job)
+                for node in job['nodes']:
+                    for rsync_compress in (True, False):
+                        self.replicator.rsync_compress = rsync_compress
+                        ret = self.replicator.sync(node, job,
+                                                   ['fake_suffix'])
+                        self.assertTrue(ret)
+                        if node['region'] != job['region']:
+                            if rsync_compress:
+                                # --compress arg should be passed to rsync
+                                # binary only when rsync_compress option is
+                                # enabled AND destination node is in a
+                                # different region
+                                self.assertTrue('--compress' in
+                                                _m_rsync.call_args[0][0])
                             else:
                                 self.assertFalse('--compress' in
                                                  _m_rsync.call_args[0][0])
-                            self.assertEqual(
-                                _m_os_path_exists.call_args_list[-1][0][0],
-                                os.path.join(job['path'], 'fake_suffix'))
-                            self.assertEqual(
-                                _m_os_path_exists.call_args_list[-2][0][0],
-                                os.path.join(job['path']))
+                        else:
+                            self.assertFalse('--compress' in
+                                             _m_rsync.call_args[0][0])
+                        self.assertEqual(
+                            _m_os_path_exists.call_args_list[-1][0][0],
+                            os.path.join(job['path'], 'fake_suffix'))
+                        self.assertEqual(
+                            _m_os_path_exists.call_args_list[-2][0][0],
+                            os.path.join(job['path']))
 
     def test_do_listdir(self):
         # Test if do_listdir is enabled for every 10th partition to rehash
